'use strict';

var fs = require('fs');
var path = require('path');
var yeoman = require('yeoman-generator');
var download = require('download');
var logSymbols = require('log-symbols');
var opn = require('opn');

var REPO = 'javascript-style';
var ZIP = 'https://github.com/stephenplusplus/' + REPO + '/archive/master.zip';

module.exports = yeoman.generators.Base.extend({
  init: function () {
    var alreadyRun = this.config.get('run');

    if (alreadyRun) {
      this._displayOptions();
    } else {
      this._plopFiles();
    }
  },

  _displayOptions: function () {
    var self = this;

    var choices = [
      {
        name: 'Install npm dependencies (jscs, jshint)',
        action: function () {
          self.npmInstall(
            [ 'jscs', 'jshint' ],
            { saveDev: true },
            self.async()
          );
        }
      },
      { name: new yeoman.inquirer.Separator() },
      {
        name: 'Visit stephenplusplus/javascript-style',
        action: opn.bind(null, 'https://github.com/stephenplusplus/' + REPO)
      },
      {
        name: 'Visit EditorConfig.org',
        action: opn.bind(null, 'http://editorconfig.org')
      },
      {
        name: 'Visit JSHint.com',
        action: opn.bind(null, 'http://www.jshint.com')
      },
      {
        name: 'Visit JavaScript Code Style Checker',
        action: opn.bind(null, 'https://github.com/mdevils/node-jscs')
      },
      {
        name: 'Visit idiomatic.js',
        action: opn.bind(null, 'https://github.com/rwaldron/idiomatic.js')
      }
    ];

    this.prompt([
      {
        name: 'next',
        type: 'list',
        message: 'Which direction shall we go?',
        choices: choices.map(function (choice) {
          return choice.name;
        })
      }
    ], function (props) {
      choices.filter(function (choice) {
        return props.next === choice.name;
      })[0].action();
    });
  },

  _plopFiles: function () {
    this.log(logSymbols.info, 'Extracting stephenplusplus/' + REPO + '...');

    download(ZIP, process.cwd(), { extract: true })
      .on('close', function (err) {
        if (err) {
          throw err;
        }

        var directory = path.join(process.cwd(), REPO + '-master');

        [ '.editorconfig', '.jshintrc', '.jscsrc', 'readme.md' ]
          .forEach(function (file) {
            var src = path.join(directory, file);
            var dest = file.replace('readme', 'codestyle');

            fs.rename(src, dest);
          }.bind(this));

        fs.rmdir(directory, function () {});

        this.config.set('run', true);

        this.log(logSymbols.success, 'Done!\n');

        this._displayOptions();
      }.bind(this));
  }
});
